; $Id: dcs_runsinfo.pro,v 1.5 2002/07/13 19:49:49 john Exp $
; DCS_RUNSINFO.PRO
;
; This program reads in and plots sample temperatures, and certain other
; parameters, for several files.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro dcs_runsinfoCleanup,tlb
widget_control,tlb,get_uvalue = pState
wdelete,(*pState).winState.winPix
ptr_free,(*pState).winState.xPtr,(*pState).winState.yPtr
ptr_free,pState
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro dcs_runsinfoQuit,event
widget_control,event.top,/destroy
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro dcs_runsinfoPlot,event
widget_control,event.top,get_uvalue = pState
widget_control,(*pState).siggroup,get_value = type
;
; Generate x and y values.
y = *(*pState).dcsStatePtr[type]
npts=n_elements(y[*,0])
x = findgen(npts)+1
;
; Generate x and y titles.
runnames=*(*pState).runnamesPtr
xt0=runnames[0]
pos0=strpos(xt0,'200',/reverse_search)
xtitle = "First run is "+strmid(xt0,pos0,11)
case type of
	0: ytitle = "Temperature (K)"
	1: ytitle = "Wavelength (A)"
	2: ytitle = "Speed ratio denominator"
	3: ytitle = "tsdmin (us)"
	4: ytitle = "Duration (h)"
	5: ytitle = "FC0 count rate (c/s)"
	6: ytitle = "He-3 count rate (c/s)"
	7: ytitle = "Status of ORC (0=stopped,1=oscillating)"
	else:
endcase
;
; Generate x and y ranges (in data units)
if (*pState).winState.autoscale eq 1 or $
	(*pState).winState.autoscale eq 2 then begin
  (*pState).winState.xrange = [0.0,float(npts)]
endif
;
if (*pState).winState.autoscale eq 1 or $
	(*pState).winState.autoscale eq 3 then begin
  dy = 0.05*(max(y)-min(y))
  if (dy eq 0.0) then dy = min(y)*0.2
  (*pState).winState.yrange = [(min(y)-dy) > 0,max(y)+dy > 1]
endif
;
xrange = (*pState).winState.xrange
yrange = (*pState).winState.yrange
;
; Initial plot with no data, no x axes.
plot,x,y(*,0),ytitle=ytitle,yrange=yrange,ystyle=1,$
	position = [0.15,0.15,0.925,0.85],/nodata,color=1,$
	xrange=xrange,xstyle=4

index=*(*pState).runindexPtr
nruns=n_elements(runnames)
;
; Define corner coordinates in data units
xl=!x.crange[0]
xr=!x.crange[1]
yb=!y.crange[0]
yt=!y.crange[1]
xw=xr-xl
yw=yt-yb
;
; Plot data for each run
if (!debug) then print,index[nruns]-1,n_elements(x)-1
case type of
	0: begin
	for k=0,nruns-1 do begin
		xx=x[index[k]]
		yy=y[index[k],0]
		if (xx gt xl and xx lt xr and yy gt yb and yy lt yt) then begin
			plots,xx,yy,psym=2,color=2
			if (index[k] lt index[k+1]-1) then begin
				xx=[x[index[k]],x[index[k+1]-1]]
				yy=[y[index[k],0],y[index[k+1]-1,0]]
				oplot,xx,yy,color=2,thick=2,psym=0
			endif
		endif
		xx=x[index[k]:index[k+1]-1]
		yy=y[index[k]:index[k+1]-1,1]
		oplot,xx,yy,color=3,thick=1,psym=-4
		yy=y[index[k]:index[k+1]-1,2]
		oplot,xx,yy,color=4,thick=1,psym=-4
	endfor
	end
	else: begin
	for k=0,nruns-1 do begin
		xx=x[index[k]]
		yy=y[index[k],0]
		if (xx gt xl and xx lt xr and yy gt yb and yy lt yt) then begin
			plots,xx,yy,psym=2,color=2
			if (index[k] lt index[k+1]-1) then begin
				xx=[x[index[k]],x[index[k+1]-1]]
				yy=[y[index[k],0],y[index[k+1]-1,0]]
				oplot,xx,yy,color=2,thick=2,psym=0
			endif
		endif
	endfor
	end
endcase
;
; Plot upper and lower x axes.
oplot,[xl,xr],[yb+yw*0.001,yb+yw*0.001],color=1,thick=1
oplot,[xl,xr],[yt-yw*0.001,yt-yw*0.001],color=1,thick=1
;
; Define x and y offsets and widths in normalized units.
xoff=!x.window[0]
xwid=!x.window[1]-!x.window[0]
yoff=!y.window[0]
ywid=!y.window[1]-!y.window[0]
;
; Plot the title.
xyouts,xoff+xwid*0.5,(yoff+ywid)+0.05,$
   xtitle,alignment = 0.5,charsize=1.5,/normal,color=1
;
; Plot x tick labels and vertical lines as x ticks.
yc=yb-0.02*yw
for j = 0,nruns-1 do begin
	runstring=strmid(runnames[j],pos0+4,7)
  xc=index[j]+1
	if (xc gt xl and xc lt xr) then begin
		xyouts,xc,yc,$
  		runstring,alignment = 0.0,charsize=1.0,$
	  	orientation = -45.0,/data,color=1
		oplot,[xc,xc],[yb,yt],color=1,thick=1.2,linestyle=1
	endif
endfor
;
; Optionally plot time labels.
if ((*pstate).showtimes) then begin
	startdates=*(*pState).startdatesPtr
	yc2=yt-0.02*yw
	for j = 0,nruns-1 do begin
		timestring=strmid(startdates[j],12,5)
	  xc=index[j]+1
		if (xc gt xl and xc lt xr) then begin
			xyouts,xc,yc2,$
		  	timestring,alignment = 0.0,charsize=1.0,$
		  	orientation = -45.0,/data,color=1
		endif
	endfor
endif
;
; For temperatures plot a legend
if (type eq 0) then begin
	descrip=["Set point","Control","Sample"]
	for j=0,2 do begin
		xs=xl+j*xw/3
		ys=yt+yw*0.02
		plots,[xs,xs+xw*0.08],[ys+yw*0.01,ys+yw*0.01],/data,color=j+2,thick=2
		xyouts,xs+xw*0.1,ys,descrip[j],$
			/data,alignment=0.0,charsize=1.2,charthick=1,color=1
	endfor
endif
;
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro dcs_runsinfoRead,event
widget_control,event.top,get_uvalue = pState

davesysvars
dcs_sysvars
;
dave_makeDavePtr,davePtr=davePtr
dcs_definedavestructure,davePtr=davePtr
;
files = dialog_pickfile(dialog_parent = event.top,title = 'Select run files',$
	path=!dcs_data_directory,/multiple_files)
if (files[0] eq '' or files[0] eq ' ') then begin
	widget_control,event.top,/destroy
	return
endif
;
nf=n_elements(files)
nofirstline=0
printinfo=0
omithisto=2
filetype="binary"
;
files=files[sort(files)]
;
progressBar = Obj_New($
	"SHOWPROGRESS",$
;	/cancelbutton,$;	(How does this work?)
	title=strcompress(string(nf))+" data files to be read ...",$
	xsize=300,$
	ysize=15,$
	message="Reading file 1",$
	color=184)
;
progressBar->Start
;
dcs_readdatafile,files[0],omithisto,printinfo,filetype,davePtr=davePtr
;
incr=100.0/nf
progressBar->Update,incr
;
comments=(*(*(*davePtr).dataStrPtr).specificPtr).comments
startdates=(*(*(*davePtr).dataStrPtr).specificPtr).start_date
stopdates=(*(*(*davePtr).dataStrPtr).specificPtr).stop_date
res_temps=where((*(*(*davePtr).dataStrPtr).specificPtr).temp_sample ge 0,ntemps)
res_cycs=where((*(*(*davePtr).dataStrPtr).specificPtr).temp_sample gt -1.05,ncycs)
case ntemps of
	; No temperature reading (no controller).
	0: begin
		t_sample=fltarr(ncycs)-100.0
		t_control=fltarr(ncycs)-100.0
		t_setpoint=fltarr(ncycs)-100.0
	end
	; One temperature reading, or else not recording temperature (values of 300).
	1: begin
		ts=(*(*(*davePtr).dataStrPtr).specificPtr).temp_sample[0]
		tc=(*(*(*davePtr).dataStrPtr).specificPtr).temp_control[0]
		tsp=(*(*(*davePtr).dataStrPtr).specificPtr).temp_setpoint
		if (ts eq 300.0 and tc eq 300.0 and tsp eq 300.0) then begin
			ts=-10.0
			tc=-10.0
			tsp=-10.0
		endif
		t_sample=ts
		t_control=tc
		t_setpoint=tsp
	end
	; Multiple temperature readings.
	else: begin
		t_sample=(*(*(*davePtr).dataStrPtr).specificPtr).temp_sample[0:ncycs-1]
		t_control=(*(*(*davePtr).dataStrPtr).specificPtr).temp_control[0:ncycs-1]
		t_setpoint=(fltarr(ncycs)+1)*(*(*(*davePtr).dataStrPtr).specificPtr).temp_setpoint
	end
endcase
n2=ncycs
wavelength=(fltarr(ncycs)+1)*(*(*(*davePtr).dataStrPtr).specificPtr).ch_wl
srdenom=(fltarr(ncycs)+1)*(*(*(*davePtr).dataStrPtr).specificPtr).ch_srdenom
tsdmin=(fltarr(ncycs)+1)*(*(*(*davePtr).dataStrPtr).specificPtr).tsdmin
duration=(fltarr(ncycs)+1)*(*(*(*davePtr).dataStrPtr).specificPtr).duration/3600.0
fc0rate=(fltarr(ncycs)+1)*(*(*(*davePtr).dataStrPtr).specificPtr).detsum[915]/$
	(*(*(*davePtr).dataStrPtr).specificPtr).duration
detrate=(fltarr(ncycs)+1)*total((*(*(*davePtr).dataStrPtr).specificPtr).detsum[0:912])/$
	(*(*(*davePtr).dataStrPtr).specificPtr).duration
collosc=(fltarr(ncycs)+1)*(*(*(*davePtr).dataStrPtr).specificPtr).coll_osc
;
if (nf gt 1) then begin
	for irun=1,nf-1 do begin
		file_path=files[irun]
		dcs_readdatafile,file_path,omithisto,printinfo,filetype,davePtr=davePtr
		progressBar->Update,irun*incr
		progressBar->SetLabel,'Reading file'+strcompress(string(irun))
		comments=[comments,(*(*(*davePtr).dataStrPtr).specificPtr).comments]
		startdates=[startdates,(*(*(*davePtr).dataStrPtr).specificPtr).start_date]
		stopdates=[stopdates,(*(*(*davePtr).dataStrPtr).specificPtr).stop_date]
		; ntemps is the number of meaningful (positive) temperature readings
		; ncycs is the number of run cycles
		res_temps=where((*(*(*davePtr).dataStrPtr).specificPtr).temp_sample gt 0.0,ntemps)
		res_cycs=where((*(*(*davePtr).dataStrPtr).specificPtr).temp_sample gt -1.05,ncycs)
		case ntemps of
			; No temperature reading (no controller).
			0: begin
				t_sample=[t_sample,fltarr(ncycs)-100.0]
				t_control=[t_control,fltarr(ncycs)-100.0]
				t_setpoint=[t_setpoint,fltarr(ncycs)-100.0]
			end
			; One temperature reading, or else not recording temperature (values of 300).
			1: begin
				ts=(*(*(*davePtr).dataStrPtr).specificPtr).temp_sample[0]
				tc=(*(*(*davePtr).dataStrPtr).specificPtr).temp_control[0]
				tsp=(*(*(*davePtr).dataStrPtr).specificPtr).temp_setpoint
				if (ts eq 300.0 and tc eq 300.0 and tsp eq 300.0) then begin
					ts=-10.0
					tc=-10.0
					tsp=-10.0
				endif
				t_sample=[t_sample,ts]
				t_control=[t_control,tc]
				t_setpoint=[t_setpoint,tsp]
			end
			; Multiple temperature readings.
			else: begin
				t_sample=[t_sample,$
					(*(*(*davePtr).dataStrPtr).specificPtr).temp_sample[0:ncycs-1]]
				t_control=[t_control,$
					(*(*(*davePtr).dataStrPtr).specificPtr).temp_control[0:ncycs-1]]
				t_setpoint=[t_setpoint,(fltarr(ncycs)+1)*$
					(*(*(*davePtr).dataStrPtr).specificPtr).temp_setpoint]
			end
		endcase
		n2=[n2,n2[irun-1]+ncycs]
		wavelength=[wavelength,$
			(fltarr(ncycs)+1)*(*(*(*davePtr).dataStrPtr).specificPtr).ch_wl]
		srdenom=[srdenom,$
			(fltarr(ncycs)+1)*(*(*(*davePtr).dataStrPtr).specificPtr).ch_srdenom]
		tsdmin=[tsdmin,$
			(fltarr(ncycs)+1)*(*(*(*davePtr).dataStrPtr).specificPtr).tsdmin]
		duration=[duration,$
			(fltarr(ncycs)+1)*(*(*(*davePtr).dataStrPtr).specificPtr).duration/3600.0]
		fc0rate=[fc0rate,$
			(fltarr(ncycs)+1)*(*(*(*davePtr).dataStrPtr).specificPtr).detsum[915]/$
				(*(*(*davePtr).dataStrPtr).specificPtr).duration]
		detrate=[detrate,$
			(fltarr(ncycs)+1)*total((*(*(*davePtr).dataStrPtr).specificPtr).detsum[0:912])/$
				(*(*(*davePtr).dataStrPtr).specificPtr).duration]
		collosc=[collosc,$
			(fltarr(ncycs)+1)*(*(*(*davePtr).dataStrPtr).specificPtr).coll_osc]
	endfor
endif
;
progressBar->Destroy
Obj_Destroy, progressBar
;
(*pState).nruns=nf
(*pState).runindexPtr=ptr_new([0,n2])
(*pState).runnamesPtr=ptr_new(files)
(*pState).runcommentsPtr=ptr_new(comments)
(*pState).startdatesPtr=ptr_new(startdates)
(*pState).stopdatesPtr=ptr_new(stopdates)
(*pState).dcsStatePtr[0] = ptr_new([[t_setpoint],[t_control],[t_sample]])
(*pState).dcsStatePtr[1] = ptr_new(wavelength)
(*pState).dcsStatePtr[2] = ptr_new(srdenom)
(*pState).dcsStatePtr[3] = ptr_new(tsdmin)
(*pState).dcsStatePtr[4] = ptr_new(duration)
(*pState).dcsStatePtr[5] = ptr_new(fc0rate)
(*pState).dcsStatePtr[6] = ptr_new(detrate)
(*pState).dcsStatePtr[7] = ptr_new((collosc eq 1)*0.999+(collosc eq 0)*0.001)
(*pState).winState.autoscale=1
;
wset,(*pState).winState.winPix
dcs_runsinfoPlot,event
wset,(*pState).winState.winVis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winState.winPix]
empty
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro dcs_runsinfoDraw,event
widget_control,event.top,get_uvalue = pState
widget_control,(*pState).siggroup,get_value = type
;
case event.type of
0:	begin		; button press
	  (*pState).winState.mouse = event.press
	  if (*pState).winState.mouse eq 4 then begin
	    (*pState).winState.autoscale = 1
	    return
	  endif
	  if (*pState).winState.mouse eq 1 then begin
	    (*pState).winState.xbox[0] = event.x
	    (*pState).winState.ybox[0] = event.y
	    wset,(*pState).winState.winVis
	    device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winState.winPix]
	    empty
	    (*pState).winState.autoscale = 0
	  endif
	end
1:	begin	; button release
	  xll = (*pState).winState.xbox[0] < (*pState).winState.xbox[1]
	  yll = (*pState).winState.ybox[0] < (*pState).winState.ybox[1]
	  w = abs((*pState).winState.xbox[1] - (*pState).winState.xbox[0])
	  h = abs((*pState).winState.ybox[1] - (*pState).winState.ybox[0])
	  xur = xll + w
	  yur = yll + h
	  ll = convert_coord(xll,yll,/device,/to_data,/double)
	  ur = convert_coord(xur,yur,/device,/to_data,/double)
	  (*pState).winState.xrange = [ll[0],ur[0]]
	  (*pState).winState.yrange = [ll[1],ur[1]]
	  wset,(*pState).winState.winPix
	  dcs_runsinfoPlot,event
	  wset,(*pState).winState.winVis
	  device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winState.winPix]
	  (*pState).winState.mouse = 0B
	end
2:	begin	; mouse motion
	  result=convert_coord(event.x,event.y,/device,/to_data,/double)
	  index=*(*pstate).runindexPtr
	  runnames=*(*pstate).runnamesPtr
	  comments=*(*pstate).runcommentsPtr
	  startdates=*(*pstate).startdatesPtr
	  stopdates=*(*pstate).stopdatesPtr
	  runnum=fix(max(where(result[0] gt index+1)))
	  if (runnum ge 0 and runnum le n_elements(runnames)-1) then begin
		  pos0=strpos(runnames[runnum],'200',/reverse_search)
			runstring=strmid(runnames[runnum],pos0+4,7)
			xcurs_info=" [Run "+runstring+": cycle"+$
				strcompress(string(result[0]-index(runnum),format="(i3)"))+$
				"]"
			case type of
				0: ycurs_info=string(result[1],format="(f8.2)")+"K"
				1: ycurs_info=string(result[1],format="(f8.3)")+"A"
				2: ycurs_info=string(result[1],format="(i3)")
				3: ycurs_info=string(result[1],format="(f8.0)")+" us"
				4: begin
					if (result[1] gt 2.0) then ycurs_info=string(result[1],$
						format="(f8.2)")+" hr" else $
						ycurs_info=string(result[1]*60.0,format="(f8.2)")+" min"
					end
				5: ycurs_info=string(result[1],format="(f8.2)")+" c/s"
				6: ycurs_info=string(result[1],format="(f8.2)")+" c/s"
				7: ycurs_info=string(result[1],format="(f8.3)")
				else:
			endcase
		  pos0=strpos(startdates[runnum],'200',/reverse_search)
		  dt1=startdates[runnum]
		  dt2=stopdates[runnum]
			f=(result[0]-1-index[runnum])/(index[runnum+1]-index[runnum])
			dtstring=dcs_interpoldate(dt1,dt2,f)
			info=[dtstring+xcurs_info+ycurs_info,comments[runnum]]
		endif else info=""
		widget_control,(*pState).curswindow,set_value=info
	  if (*pState).winState.mouse eq 1 then begin
	  	(*pState).winState.xbox[1] = event.x
	  	(*pState).winState.ybox[1] = event.y
	  	xc = [(*pState).winState.xbox[0],event.x,event.x,(*pState).winState.xbox[0],$
	  	      (*pState).winState.xbox[0]]
	  	yc = [(*pState).winState.ybox[0],(*pState).winState.ybox[0],event.y,event.y,$
	  	      (*pState).winState.ybox[0]]
	  	wset,(*pState).winState.winVis
	  	device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winState.winPix]
	  	plots,xc,yc,/device,color=1
	  	empty
	  endif
	end
else:
endcase
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro dcs_runsinfoPrint,event
thisDevice = !d.name
deviceKeywords = ps_form(Cancel = canceled,parent = event.top)
if canceled ne 1 then begin
  SET_PLOT, 'PS'
  DEVICE, _Extra = deviceKeywords
  dcs_runsinfoPlot,event
  DEVICE, /CLOSE_FILE
  set_plot,thisDevice
endif
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro dcs_runsinfo_event,event
widget_control,event.top,get_uvalue = pState
if event.id eq (*pState).siggroup then begin
  (*pState).winState.autoscale = 3
	widget_control,event.top,set_uvalue = pState
  wset,(*pState).winState.winPix
  dcs_runsinfoPlot,event
  wset,(*pState).winState.winVis
  device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winState.winPix]
  empty
endif
if event.id eq (*pState).showbox then begin
	if ((*pState).showtimes eq 1) then begin
		(*pState).showtimes=0
		widget_control,(*pstate).showbox,set_value="Show times"
	endif else begin
		(*pState).showtimes=1
		widget_control,(*pstate).showbox,set_value="Hide times"
	endelse
	widget_control,event.top,set_uvalue = pState
  wset,(*pState).winState.winPix
  dcs_runsinfoPlot,event
  wset,(*pState).winState.winVis
  device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winState.winPix]
  empty
endif
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro dcs_runsinfo,event
; Widget definition module
;
; To add a new quantity to the list of those that can be displayed, make changes
; (1) in the definition of 'sigtypes' (below)
; (2) to the size of dcsStatePtr (below)
; (3) in dcs_runsinfoRead to the initial value and to subsequent values
;	(4) in dcs_runsinfoRead define the new element in (*pState).dcsStatePtr
; (5) in dcs_runsinfoDraw to the value of ycurs_info
; (6) in dcs_runsinfoPlot to define the y label.

daveDeSensitizeButtons,event

amcolor = {color, $ ; name
red:   [255,0,255,  0,  0,  0,255,255,152,128,128,128,255,178,255],$ ; red values
green: [255,0,  0,255,  0,255,  0,255,120,128,128,128, 54,192,128],$ ; green values
blue:  [255,0,  0,  0,255,255,255,  0,103,220, 60,128, 54,220,128] $ ; blue values
}
colorset
oldct=dcs_getctinfo()
tvlct,amcolor.red,amcolor.green,amcolor.blue
;
tlb = widget_base(/row,title = 'DCS Run Information Display',mbar = bar)
filemenu = widget_button(bar,value = 'File',/menu)
void = widget_button(filemenu,value = 'Select raw data files',$
	event_pro = 'dcs_runsinfoRead')
void = widget_button(filemenu,value = 'Print to a postscript file',$
       event_pro = 'dcs_runsinfoPrint')
void = widget_button(filemenu,value = 'Quit',event_pro = 'dcs_runsinfoQuit')
base1 = widget_base(tlb,/col)
sigtypes = ['Temperatures','Wavelength','S. R. denominator','tsdmin',$
	'Duration','FC0 rate','3He detector rate','Status of ORC']
siggroup = cw_bgroup(base1, sigtypes, /col, /exclusive,$
         label_top='Selection',/no_release,$
         frame=1,set_value=0,/return_index)
showbox = widget_button(base1,value='Show times')
base2 = widget_base(tlb,/col)
base3 = widget_base(base2,/align_center)
curswindow =widget_text(base3,xsize=50,ysize=2,/editable,font="COURIER*14")
xsize = !screensize[0]*0.8 & ysize = !screensize[1]*0.7
win = widget_draw(base2,xsize = xsize,ysize = ysize,/button_events,$
      event_pro = 'dcs_runsinfoDraw',/motion_events)

centertlb,tlb
widget_control,tlb,/realize
widget_control,win,get_value = winVis
window,/free,/pixmap,xsize = xsize,ysize = ysize
winPix = !d.window

winState =  {winPix:winPix,$
             winVis:winVis,$
             win:win,$
             xbox:[0.0,1.0],$
             xrange:[0.0d0,1.0d0],$
             ybox:[0.0,1.0],$
             yrange:[0.0d0,1.0d0],$
             xPtr:ptr_new(/allocate_heap),$
             yPtr:ptr_new(/allocate_heap),$
             xtitle:'',$
             ytitle:'',$
             title:'',$
             mouse:0B,$
             autoscale:1}

state = {curswindow:curswindow,$
         winState:winState,$
         dcsStatePtr:ptrarr(8,/allocate_heap),$
         nruns:1,$
         showbox:showbox,$
         showtimes:0,$
         siggroup:siggroup,$
         runindexPtr:ptr_new(/allocate_heap),$
         runnamesPtr:ptr_new(/allocate_heap),$
         runcommentsPtr:ptr_new(/allocate_heap),$
         startdatesPtr:ptr_new(/allocate_heap),$
         stopdatesPtr:ptr_new(/allocate_heap)}

widget_control,tlb,set_uvalue = ptr_new(state,/no_copy)
dcs_runsinfoRead,{event,id:void,top:tlb,handler:0l}
if (widget_info(tlb,/valid_id)) then $
	xmanager,'dcs_runsinfo',tlb,cleanup = 'dcs_runsinfoCleanup'

daveSensitizeButtons,event
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
