; $Id: fans_plotdata.pro,v 1.4 2002/07/16 16:09:31 azuah Exp $
;===============================================================================
; Helper functions for FANS_DATREDUC program defined in fans_datreduc.pro.
;
; * Plot fans data on the display window
;
; Richard Tumanjong Azuah
;
; May 23 2002 - Initial writing
; 
;===============================================================================


;===============================================================================
; maintains a list of detectors to be masked from the output data
pro fans_updMskList, sPtr, detnos, err=err,emsg=emsg
err=0 & emsg=''

; one input parameters necessary
if (n_params() ne 2) then begin
    err = 1
    emsg = 'In "fans_updMskList" - Invalid number of parameters'
    return
endif

; check input parameters are defined
if ( (n_elements(detnos) le 0)  or (ptr_valid(sPtr) eq 0) ) then begin
    err = 2
    emsg = 'In "fans_updMskList" - Input parameter(s) undefined'
    return
endif

; grap the current mask list
if  ptr_valid((*sPtr).mskListPtr) then list = (*(*sPtr).mskListPtr)
ptr_free,(*sPtr).mskListPtr     ; clean (empty) the current list

lsize = n_elements(list)
if (lsize gt 0) then begin      ; list exists
    result = where(list eq detnos)
    if (result[0] ge 0) then begin ; detnos is in the list already - remove it
        lsize = lsize - 1
        if (result[0] le lsize-1) then $
          for i = result[0], lsize-1 do list[i] = list[i+1]
    endif else begin            ; detnos not in list - add it
        lsize = lsize + 1
        list = [list,detnos]
    endelse
endif else begin                ; no list yet - create one
    lsize = 1
    list = [detnos]
endelse 

if (lsize ge (*(*(*sPtr).sDataPtr)[(*sPtr).sampIndex]).ndets-2) then begin ; removing all detectors!!!
    message = ['Attempting to mask all detectors.', $
               'No output can be generated!']
    void = dialog_message(message,/information,dialog_parent=(*sPtr).tlbID)
endif


; update state variable
lstring = ''
; mskOnID should be desensitized and off (ie deselected) by default unless there
; are actually detectors to be masked.
widget_control, (*sPtr).mskOnID, get_uvalue=mskon ; get current msk status (ie whether on/off)
widget_control, (*sPtr).mskOnID, sensitive=0 ; should be desensitized
if (mskon) then $               ; if on, switch it off! but remember it was on
  widget_control, (*sPtr).mskOnID, set_button=0, set_uvalue=0

if (lsize gt 0) then begin
    widget_control, (*sPtr).mskOnID, sensitive=1
    if (mskon) then $           ; if it was on previously, switch it back on
      widget_control, (*sPtr).mskOnID, set_button=1, set_uvalue=mskon
    newlist = list[0:lsize-1]
    newlist = newlist[sort(newlist)] ; sort in ascending order
    (*sPtr).mskListPtr = ptr_new(newlist)
    strarr = strtrim(string(newlist),1) 
    for i = 0,lsize-1 do $
      lstring = lstring + strarr[i] + ' '
endif

; switch on output calculation flag
(*sPtr).doOutpCalc = 1

; indicates Fast bkgd fit has to be recalculated
(*sPtr).updFast = 1


; update list in mask widget
widget_control, (*sPtr).mskListID, set_value=lstring

end


;===============================================================================
pro fans_plotMskList, sPtr, err=err, emsg=emsg
err=0 & emsg=''

; one input parameters necessary
if (n_params() ne 1) then begin
    err = 1
    emsg = 'In "fans_plotMskList" - Invalid number of parameters'
    return
endif

; check input parameters are defined
if (ptr_valid(sPtr) eq 0) then begin
    err = 2
    emsg = 'In "fans_plotMskList" - Input parameter undefined'
    return
endif

if ( ptr_valid((*sPtr).mskListPtr) eq 0) then return

list = (*(*sPtr).mskListPtr)

lsize = n_elements(list)
ymin = !y.crange[0]  &  ymax = !y.crange[1]
yvals = [ymin,ymin,ymax,ymax]
for i = 0, lsize-1 do begin
    xmin = list[i]-0.50  &  xmax=list[i]+0.50
    xvals = [xmin,xmax,xmax,xmin]
    polyfill,xvals,yvals
endfor


end


;===============================================================================
; event handler for button events dispatched from the plot window. The
; design is such that the plot window only generates events when samp
; or bkgd data are being viewed.
pro fans_plotClick, event

; deal only with a realease event of the left-mouse button
if (event.release ne 1) then return

; if the x coord of mouse occurs outside the detector range, ignore it
xydat = convert_coord(event.x,event.y,/device,/to_data)
detnos = round(xydat[0])
widget_control, event.top, get_uvalue=sPtr
lolim=2.5
hilim=50.49
if (ptr_valid((*sPtr).sDataPtr)) then begin
    hilim = (*(*(*sPtr).sDataPtr)[0]).ndets + 0.49
endif
if (detnos lt lolim or detnos gt hilim) then return

; add/remove det from mask list if it isn't/is there followed by an
; update to the mask widget
fans_updMskList, sPtr,detnos

; refresh plot window - but need to find out fist what data type to
;                       plot. Only samp and bkgd plots can be called
;                       from here.
widget_control, (*sPtr).sViewID, get_uvalue=svOn
widget_control, (*sPtr).bViewID, get_uvalue=bvOn
if svOn then dataType = 'samp'
if bvOn then dataType = 'bkgd'
fans_plotData, sPtr, dataType, err=err,emsg=emsg
if (err ne 0) then begin
    printerr,err,emsg
    return
endif

end





;===============================================================================
; given a 1D 'array', this function returns the index of the element
; which is closest to the input 'val'.
function locateIndex, array,val

nspec = size(array,/n_elements)
result = where(array gt val)
if (result[0] lt 0) then begin  ; ie if where function returns null result
    index = nspec-1             ; ==> use last index
endif else begin
    index = result[0]
    diff1 = abs(val - array[index-1]) 
    diff2 = abs(array[index] - val)
    if (diff1 le diff2) then index = index - 1 ; use index of element closer to val!
endelse

return, index

end



;===============================================================================
; Slider values are continuous. However, the energy values that the
; slider represents only have certain values given by the energy
; array. This routine ensures that the slider only sets/displays valid
; energy values that are present in the energy array. This routine
; must be called whenever the data being displayed changes.
pro fans_setSliderValue, wid, energyPtr, dvalue, err=err, emsg=emsg

err=0  &  emsg = ''

; one input parameters necessary
if (n_params() lt 2) then begin
    err = 1
    emsg = 'In "fans_setSliderValue" - Invalid number of parameters'
    return
endif

; check input parameters are defined
if ( (n_elements(wid) le 0)  or (ptr_valid(energyPtr) eq 0) ) then begin
    err = 2
    emsg = 'In "fans_setSliderValue" - Input parameter(s) undefined'
    return
endif

; if input parameter dvalue missing, use current slider value instead
if (n_elements(dvalue) le 0) then $
  widget_control, wid, get_value=dvalue ;dvalue = fslider_get_value(wid) 

; now locate the closest entry to dvalue in the energy array
index = locateIndex((*energyPtr),dvalue)

; update slider value with this 'proper' energy 
widget_control, wid, set_value=(*energyPtr)[index]

; save this index in the uvalue of the widget
widget_control, wid, set_uvalue=index

end



;===============================================================================
pro fans_syncSlider, wid, energyPtr, reset, err=err, emsg=emsg

err=0  &  emsg = ''
; two input parameters necessary
if (n_params() ne 3) then begin
    err = 1
    emsg = 'In "syncSlider" - Invalid number of procedure parameters'
    return
endif

; check input parameters are defined
if ((n_elements(wid) le 0)or(ptr_valid(energyPtr) eq 0)or(n_elements(reset) eq 0)) then begin
    err = 2
    emsg = 'In "syncSlider" - Input parameter(s) undefined'
    return
endif

info = size(*energyPtr)
if (info[0] ne 1) then begin
    err = 3
    emsg = 'In "syncSlider" - Input parameter has wrong dimension'
    return
endif

dmin = min(*energyPtr, max=dmax)
if (reset gt 0) then begin
    ;; set slider range to be same as energy array. Initialise it to its
    ;; min value
    range = [dmin,dmin,dmax]
    widget_control, wid, set_value = range
    fans_setSliderValue, wid, energyPtr, dmin, err=err, emsg=emsg
endif else begin
    widget_control, wid, get_value=dvalue ;dvalue = fslider_get_value(wid)
    if (dvalue lt dmin) then dvalue = dmin
    if (dvalue gt dmax) then dvalue = dmax
    range = [dvalue,dmin,dmax]
    widget_control, wid, set_value = range
    fans_setSliderValue, wid, energyPtr, davlue, err=err,emsg=emsg
endelse

end



;===============================================================================
; Event handler for the Energy Cut Slider widget 
pro fans_sliderEvent, event

widget_control,event.top,get_uvalue=sPtr

; determine which type of data is being displayed. Note that it only
; make sense to view energy slices for sample and bkgd data. Cuts for
; Fast bkgd and output data are not implemented

widget_control, (*sPtr).sViewID, get_uvalue=svOn
widget_control, (*sPtr).bViewID, get_uvalue=bvOn
;widget_control, (*sPtr).oViewID, get_uvalue=ovOn

if svOn then dataType = 'samp'
if bvOn then dataType = 'bkgd'
;if ovOn then dataType = 'outp'

if ( n_elements(dataType) le 0) then begin
    if (!debug) then print,'Improper behaviour within "fans_sliderEvent"'
    return
endif

; get the energy array of current data
switch dataType of

    'samp': begin
        index = (*sPtr).sampIndex
        fdStructPtr = (*(*sPtr).sDataPtr)[index] ; get a ptr to the approp. fans data struct
        break
    end

    'bkgd': begin
        index = (*sPtr).bkgdIndex
        fdStructPtr = (*(*sPtr).bDataPtr)[index] ; get a ptr to the approp. fans data struct
        break
    end

;    'outp': begin
;        print,'To be implemented...'
;        return
;        index = (*sPtr).outpIndex
;        fdStructPtr = (*(*sPtr).oDataPtr)[index] ; get a ptr to the approp. fans data struct
;        break
;    end


endswitch

energyPtr = (*fdStructPtr).energyPtr

; now alter the slider value so it is a valid value within the energy array.
fans_setSliderValue,(*sPtr).eSliderID,energyPtr,err=err,emsg=emsg
if (err ne 0) then begin
    printerror,err,emsg
    return
endif

; finally, update the plot.
fans_plotdata,sPtr,dataType

end



;===============================================================================
pro fans_plotData, sPtr, dataType, err=err,emsg=emsg

err = 0  &  emsg = ''

; two input parameters necessary
if (n_params() ne 2) then begin
    err = 1
    emsg = '"In fans_plotData" - Invalid number of procedure parameters'
    return
endif;===============================================================================


; check input parameters are defined
if ( (n_elements(dataType) le 0)  or (ptr_valid(sPtr) eq 0) ) then begin
    err = 2
    emsg = 'In "fans_plotData" - Input parameter(s) undefined'
    return
endif


switch dataType of

    'samp': begin
        index = (*sPtr).sampIndex
        fdStructPtr = (*(*sPtr).sDataPtr)[index] ; get a ptr to the approp. fans data struct
;        omega = (*(*fdStructPtr).energyPtr) ; energy tranfers
;        totCnts = (*(*fdStructPtr).totCntsPtr) ; total counts
        date = (*fdStructPtr).date
        title = (*fdStructPtr).title
        subtitle = 'Expt date: '+date
        xtitle = 'Detector number'
        ytitle = 'Raw Counts'
        ;; for now, just plot total counts vs omega
;        plot,omega,totCnts, $
;             xstyle=1, $
;             xtitle=xtitle,subtitle=subtitle,ytitle=ytitle,title=title

        ;; get index of energy value set by slider
        widget_control, (*sPtr).eSliderID, get_uvalue=eIndex
        
        ;; get detector counts for this energy
        ;; NB dets 1 and 2 are monitors!
        ndets = (*fdStructPtr).ndets
        nspec = (*fdStructPtr).nspec
        if ( (eIndex lt 0) or (eIndex ge nspec) ) then eIndex=0 
        detCnts = (*(*fdStructPtr).detCntsPtr)[2:ndets-1,eIndex] ; ignore monitors
        detNos = indgen(ndets-2)+3
        
        ;; plot detector counts vs detector number
        plot,detNos,detCnts, $
             xstyle=1,/ynozero,psym=-4,symsize=1.5,linestyle=0,charsize=1.4, $
             xtitle=xtitle,subtitle=subtitle,ytitle=ytitle,title=title, $
             xrange=[0,ndets+2]
        ;; plot errors
;        if ( size(where(detCnts lt 0),/dimensions) eq 0) then begin ; -ve nos are not allowed 
            sigma = (*(*fdStructPtr).detErrsPtr)[2:ndets-1,eIndex] ;0.5*sqrt(detCnts)
            errplot, detNos, detCnts-sigma, detCnts+sigma ; plot errors
;        endif

        ;; display masked detectors
        fans_plotMskList, sPtr

        break
    end

    'bkgd': begin
        index = (*sPtr).bkgdIndex
        fdStructPtr = (*(*sPtr).bDataPtr)[index] ; get a ptr to the approp. fans data struct
        date = (*fdStructPtr).date
        title = (*fdStructPtr).title
        subtitle = 'Expt date: '+date
        xtitle = 'Detector number'
        ytitle = 'Raw Counts'

        ;; get index of energy value set by slider
        widget_control, (*sPtr).eSliderID, get_uvalue=eIndex
        
        ;; get detector counts for this energy
        ;; NB dets 1 and 2 are monitors!
        ndets = (*fdStructPtr).ndets
        nspec = (*fdStructPtr).nspec
        if ( (eIndex lt 0) or (eIndex ge nspec) ) then eIndex=0 
        detCnts = (*(*fdStructPtr).detCntsPtr)[2:ndets-1,eIndex] ; ignore monitors
        detNos = indgen(ndets-2)+3
        
        ;; plot detector counts vs detector number
        plot,detNos,detCnts, $
             xstyle=1,/ynozero,psym=-4,symsize=1.5,linestyle=0,charsize=1.4, $
             xtitle=xtitle,subtitle=subtitle,ytitle=ytitle,title=title, $
             xrange=[0,ndets+2]
        ;; plot errors
;        if ( size(where(detCnts lt 0),/dimensions) eq 0) then begin ; -ve nos are not allowed 
            sigma = (*(*fdStructPtr).detErrsPtr)[2:ndets-1,eIndex] ;0.5*sqrt(detCnts)
            errplot, detNos, detCnts-sigma, detCnts+sigma ; plot errors
;        endif

        ;; display masked detectors
        fans_plotMskList, sPtr
        
        break
    end

    'fast': begin
        index = (*sPtr).fastIndex
        fdStructPtr = (*(*sPtr).fDataPtr)[index] ; get a ptr to the approp. fans data struct
        omega = (*(*fdStructPtr).energyPtr) ; energy tranfers
        totCnts = (*(*fdStructPtr).totCntsPtr) ; total counts
        date = (*fdStructPtr).date
        title = (*fdStructPtr).title
        subtitle = 'Expt date: '+date
        xtitle = 'Energy Transfer (meV)'
        ytitle = 'Raw Counts'
        ;; for now, just plot total counts vs omega
        ;;        plot,omega,totCnts, $
        ;;             xstyle=1, $
        ;;             xtitle=xtitle,subtitle=subtitle,ytitle=ytitle,title=title
        plot,omega,totCnts, $
             xstyle=1,ystyle=1,psym=-4,symsize=1.5,linestyle=0,charsize=1.4, $
             xtitle=xtitle,subtitle=subtitle,ytitle=ytitle,title=title
        ;; plot errors
;        if ( size(where(totCnts lt 0),/dimensions) eq 0) then begin ; -ve nos are not allowed 
;            sigma = 0.5*sqrt(totCnts)
        sigma = (*(*fdStructPtr).totErrsPtr)
        errplot, omega, totCnts-sigma, totCnts+sigma ; plot errors
;        endif

        
        break
    end

    'outp': begin
        index = (*sPtr).outpIndex
        fdStructPtr = (*(*sPtr).oDataPtr)[index] ; get a ptr to the approp. fans data struct
        monCnts = (*fdStructPtr).monitor * (*fdStructPtr).prefactor
        omega = (*(*fdStructPtr).energyPtr) ; energy tranfers
        totCnts = (*(*fdStructPtr).totCntsPtr) ; total counts
        date = (*fdStructPtr).date
        title = (*fdStructPtr).title
        subtitle = 'Expt date: '+date
        xtitle = 'Energy Transfer (meV)'
        ytitle = 'Counts per '+strtrim(string(monCnts),2)+' mon'
        ;; for now, just plot total counts vs omega
        ;;        plot,omega,totCnts, $
        ;;             xstyle=1, $
        ;;             xtitle=xtitle,subtitle=subtitle,ytitle=ytitle,title=title
        plot,omega,totCnts, $
             xstyle=1,ystyle=1,psym=-4,symsize=1.0,linestyle=0,charsize=1.4, $
             xtitle=xtitle,subtitle=subtitle,ytitle=ytitle,title=title
        ;; plot errors
;        negNos = where(totCnts lt 0)
;        if ( negNos[0] lt 0) then begin ; -ve nos are not allowed 
;            sigma = 0.5*sqrt(totCnts)
        sigma = (*(*fdStructPtr).totErrsPtr)
        errplot, omega, totCnts-sigma, totCnts+sigma ; plot errors
;        endif

        break
    end
    
endswitch

end
